@@ -34,7 +34,7 @@
 #include "launcher.h"
 
 int
-panel_ditem_launch (const GnomeDesktopItem       *item,
+panel_ditem_launch (GnomeDesktopItem             *item,
 		    GList                        *file_list,
 		    GnomeDesktopItemLaunchFlags   flags,
 		    GdkScreen                    *screen,
@@ -201,6 +201,38 @@ panel_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
 	return list;
 }
 
+GSList *
+panel_g_slist_make_unique (GSList       *list,
+			   GCompareFunc  compare,
+			   gboolean      free_data)
+{
+	GSList *sorted, *l;
+
+	g_assert (compare != NULL);
+
+	if (!list)
+		return NULL;
+
+	sorted = g_slist_copy (list);
+	sorted = g_slist_sort (sorted, compare);
+
+	for (l = sorted; l; l = l->next) {
+		GSList *next;
+
+		next = l->next;
+		if (l->data && next && next->data)
+			if (!compare (l->data, next->data)) {
+				list = g_slist_remove (list, l->data);
+				if (free_data)
+					g_free (l->data);
+			}
+	}
+
+	g_slist_free (sorted);
+
+	return list;
+}
+
 static GtkWidget *
 panel_dialog (GdkScreen  *screen,
 	      int         type,
@@ -574,7 +606,8 @@ panel_load_icon (GtkIconTheme  *icon_theme,
 							    &error);
 
 		if (error) {
-			*error_msg = g_strdup (error->message);
+			if (error_msg)
+				*error_msg = g_strdup (error->message);
 			g_error_free (error);
 		}
 	}
