@@ -18,6 +18,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <libbonobo.h>
+#include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -29,7 +30,7 @@
 #include "drawer.h"
 #include "button-widget.h"
 #include "launcher.h"
-#include "menu.h"
+#include "panel-context-menu.h"
 #include "panel-util.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
@@ -147,9 +148,17 @@ void
 back_change (AppletInfo  *info,
 	     PanelWidget *panel)
 {
-	if (info->type == PANEL_OBJECT_BONOBO)
+	switch (info->type) {
+	case PANEL_OBJECT_BONOBO:
 		panel_applet_frame_change_background (
 			PANEL_APPLET_FRAME (info->widget), panel->background.type);
+		break;
+	case PANEL_OBJECT_MENU_BAR:
+		panel_menu_bar_change_background (PANEL_MENU_BAR (info->widget));
+		break;
+	default:
+		break;
+	}
 }
 
 static void
@@ -219,7 +228,8 @@ deactivate_idle (gpointer data)
 }
 
 static void
-menu_deactivate(GtkWidget *w, PanelData *pd)
+context_menu_deactivate (GtkWidget *w,
+			 PanelData *pd)
 {
 	if (pd->deactivate_idle == 0)
 		pd->deactivate_idle = g_idle_add (deactivate_idle, pd);
@@ -227,6 +237,13 @@ menu_deactivate(GtkWidget *w, PanelData *pd)
 	panel_toplevel_pop_autohide_disabler (PANEL_TOPLEVEL (pd->panel));
 }
 
+static void
+context_menu_show (GtkWidget *w,
+		   PanelData *pd)
+{
+	panel_toplevel_push_autohide_disabler (PANEL_TOPLEVEL (pd->panel));
+}
+
 static void
 panel_recreate_context_menu (PanelData *pd)
 {
@@ -278,10 +295,15 @@ static GtkWidget *
 panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
 	if (!pd->menu) {
-		pd->menu = g_object_ref (create_panel_context_menu (panel));
-		gtk_object_sink (GTK_OBJECT (pd->menu));
-		g_signal_connect (pd->menu, "deactivate",
-				  G_CALLBACK (menu_deactivate), pd);
+		pd->menu = g_object_ref (panel_context_menu_create (panel));
+		if (pd->menu) {
+			gtk_object_sink (GTK_OBJECT (pd->menu));
+			g_signal_connect (pd->menu, "deactivate",
+					  G_CALLBACK (context_menu_deactivate),
+					  pd);
+			g_signal_connect (pd->menu, "show",
+					  G_CALLBACK (context_menu_show), pd);
+		}
 	}
 
 	return pd->menu;
@@ -332,8 +354,8 @@ panel_popup_menu (PanelToplevel *toplevel,
 		panel_data->insertion_pos = -1;
 	
 	menu = make_popup_panel_menu (panel_widget);
-
-	panel_toplevel_push_autohide_disabler (toplevel);
+	if (!menu)
+		return FALSE;
 
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_window_get_screen (GTK_WINDOW (toplevel)));
@@ -458,7 +480,7 @@ drop_url (PanelWidget *panel,
 
 	panel_launcher_create_from_info (panel->toplevel, position, FALSE,
 					 netscape_url[NETSCAPE_URL_URL],
-					 name, comment, "gnome-globe.png");
+					 name, comment, "gnome-globe");
 
 	g_free (comment);
 	g_strfreev (netscape_url);
@@ -469,18 +491,19 @@ drop_url (PanelWidget *panel,
 static gboolean
 drop_menu (PanelWidget *panel,
 	   int          position,
+	   const char  *menu_filename,
 	   const char  *menu_path)
 {
 	if (!panel_profile_id_lists_are_writable ())
 		return FALSE;
 
-	panel_menu_button_create (panel->toplevel,
-				  position,
-				  menu_path,
-				  menu_path != NULL,
-				  NULL);
+	return panel_menu_button_create (panel->toplevel,
+					 position,
+					 menu_filename,
+					 menu_path,
+					 menu_path != NULL,
+					 NULL);
 
-	return TRUE;
 }
 
 static gboolean
@@ -490,17 +513,24 @@ drop_uri (PanelWidget *panel,
 	  const char  *icon)
 {
 	char  *base;
+	char  *name;
+	char  *comment;
 
 	if (!panel_profile_id_lists_are_writable ())
 		return FALSE;
 
 	base = g_path_get_basename (uri);
-
-	panel_launcher_create_from_info (
-		panel->toplevel, position, FALSE, uri, base, uri, icon);
-
+	name = gnome_vfs_unescape_string_for_display (base);
 	g_free (base);
 
+	comment = gnome_vfs_unescape_string_for_display (uri);
+
+	panel_launcher_create_from_info (panel->toplevel, position, FALSE,
+					 uri, name, comment, icon);
+
+	g_free (name);
+	g_free (comment);
+
 	return TRUE;
 }
 
@@ -581,19 +611,24 @@ drop_urilist (PanelWidget *panel,
 					success = FALSE;
 				g_free (filename);
 			} else {
-				const char *icon;
+				char *icon;
+
+				icon = gnome_icon_lookup (gtk_icon_theme_get_default (),
+							  NULL, NULL, NULL,
+							  info, info->mime_type,
+							  GNOME_ICON_LOOKUP_FLAGS_NONE,
+							  NULL);
 
-				icon = NULL;
-				if (info->mime_type)
-					icon = gnome_vfs_mime_get_icon (info->mime_type);
 				if (!icon)
-					icon = "gnome-unknown.png";
+					icon = g_strdup ("gnome-unknown");
 
 				if (!drop_uri (panel, pos, uri, icon))
 					success = FALSE;
+
+				g_free (icon);
 			}
 		} else {
-			if (!drop_uri (panel, pos, uri, "gnome-unknown.png"))
+			if (!drop_uri (panel, pos, uri, "gnome-unknown"))
 				success = FALSE;
 		}
 
@@ -691,12 +726,27 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 			g_warning ("Only MOVE supported for menus/drawers");
 		success = move_applet (panel, pos, applet_index);
 
-	} else if (strncmp (applet_type, "MENU:", strlen("MENU:")) == 0) {
-		const char *menu = &applet_type[strlen ("MENU:")];
-		if (strcmp (menu, "MAIN") == 0)
-			success = drop_menu (panel, pos, NULL);
-		else
-			success = drop_menu (panel, pos, menu);
+	} else if (strncmp (applet_type, "MENU:", strlen ("MENU:")) == 0) {
+		const char *menu;
+		const char *menu_path;
+
+		menu = &applet_type[strlen ("MENU:")];
+		menu_path = strchr (menu, '/');
+
+		if (!menu_path) {
+			if (strncmp (menu, "MAIN", strlen ("MAIN")) == 0)
+				success = drop_menu (panel, pos, NULL, NULL);
+			else
+				success = drop_menu (panel, pos, menu, NULL);
+		} else {
+			char *menu_filename;
+
+			menu_filename = g_strndup (menu, menu_path - menu);
+			*menu_path++;
+			success = drop_menu (panel, pos,
+					     menu_filename, menu_path);
+			g_free (menu_filename);
+		}
 
 	} else if (!strcmp (applet_type, "DRAWER:NEW")) {
 		if (panel_profile_id_lists_are_writable ()) {
@@ -909,7 +959,6 @@ drag_motion_cb (GtkWidget	   *widget,
 	do_highlight (widget, TRUE);
 
 	panel_toplevel_unhide (toplevel);
-	panel_toplevel_queue_auto_hide (toplevel);
 
 	return TRUE;
 }
@@ -938,7 +987,12 @@ drag_leave_cb (GtkWidget	*widget,
 	       guint             time,
 	       Launcher         *launcher)
 {
+	PanelToplevel *toplevel;
+
 	do_highlight (widget, FALSE);
+
+	toplevel = PANEL_TOPLEVEL (widget);
+	panel_toplevel_queue_auto_hide (toplevel);
 }
 
 void
@@ -978,7 +1032,8 @@ panel_receive_dnd_data (PanelWidget      *panel,
 		}
 		break;
 	case TARGET_DIRECTORY:
-		success = drop_uri (panel, pos, (char *)selection_data->data, "gnome-fs-directory.png");
+		success = drop_uri (panel, pos, (char *)selection_data->data,
+				    "gnome-fs-directory");
 		break;
 	case TARGET_APPLET:
 		if (!selection_data->data) {
@@ -1198,16 +1253,19 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 {
 
 	GtkWidget *dialog;
-	char *text;
+	char *text1;
+	char *text2;
 	char *title;
 
 	if (panel_toplevel_get_is_attached (toplevel)) {
-		text = _("When a drawer is deleted, the drawer and its\n"
-			 "settings are lost. Delete this drawer?");
+		text1 = _("Delete this drawer?");
+		text2 = _("When a drawer is deleted, the drawer and its\n"
+			 "settings are lost.");
 		title = _("Delete Drawer");
 	} else {
-		text = _("When a panel is deleted, the panel and its\n"
-			 "settings are lost. Delete this panel?");
+		text1 = _("Delete this panel?");
+		text2 = _("When a panel is deleted, the panel and its\n"
+			 "settings are lost.");
 		title = _("Delete Panel");
 	}
 
@@ -1216,7 +1274,10 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 			GTK_DIALOG_MODAL,
 			GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_NONE,
-			text);
+			text1);
+	
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                          text2);	
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
